/* grdesktop - gtk rdesktop frontend
 * Copyright (C) 2002 Thorsten Sauter <tsauter@gmx.net>
 *
 * $Id: options.c,v 1.89 2004/03/30 12:31:45 tsauter Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include "options.h"

void fillRdpProtocols() {
	rdp_protocols = g_list_append(rdp_protocols, g_strdup(_("Windows NT/2000")));
	rdp_protocols = g_list_append(rdp_protocols, g_strdup(_("Windows XP/2003")));
}

void loadRdpProtocols(GtkWidget *widget) {
	if((g_list_length(rdp_protocols)-1) < iSHASH("rdp_protocol")) {
		g_warning("Unknown RDP-Protocol: %d\n", iSHASH("rdp_protocol")+4);
		setBHASH("rdp_protocol", 0);
	}

	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), iSHASH("rdp_protocol"));
}

void fillScreens() {
	gint width, height;

	width = gdk_screen_width();
	height = gdk_screen_height();

	if(checkScreen("640x480", width, height))
		screensize = g_list_append(screensize, "640x480");
	if(checkScreen("800x600", width, height))
		screensize = g_list_append(screensize, "800x600");
	if(checkScreen("1024x768", width, height))
		screensize = g_list_append(screensize, "1024x768");
	if(checkScreen("1270x940", width, height))
		screensize = g_list_append(screensize, "1270x940");
	if(checkScreen("1280x1024", width, height))
		screensize = g_list_append(screensize, "1280x1024");
	if(checkScreen("1600x1280", width, height))
		screensize = g_list_append(screensize, "1600x1280");

	screensize = g_list_append(screensize, _("Fullscreen"));
}

gint checkScreen(const gchar *geo, gint width, gint height) {
	gchar **split;
	gint iwidth, iheight;

	split = g_strsplit(geo, "x", 2);
	iwidth = atoi(split[0]);
	iheight = atoi(split[1]);

	if(split != NULL)
		g_strfreev(split);

	if((iwidth < width) && (iheight < height))
		return(1);

	return(0);
}

void loadScreens(GtkAdjustment *widget) {
	gint i, count;
	gboolean found;
	gchar *item = NULL;

	found = FALSE;
	i = 0;

	if(SHASH("geometry") != NULL) {
		count = g_list_length(screensize);
		for(i=0; i<count; i++) {
			item = g_list_nth_data(screensize, i);
			if(!g_strcasecmp(SHASH("geometry"), item)) {
				found = TRUE;
				break;
			}
		}
	}

	if(found == TRUE)
		gtk_adjustment_set_value(GTK_ADJUSTMENT(widget), i);
	else
		gtk_adjustment_set_value(GTK_ADJUSTMENT(widget), 0);
}

void fillColors() {
	colors = g_list_append(colors, g_strdup(_("256 Colors")));
	colors = g_list_append(colors, g_strdup(_("High Color (15bit)")));
	colors = g_list_append(colors, g_strdup(_("High Color (16bit)")));
	colors = g_list_append(colors, g_strdup(_("True Color (24bit)")));
}

void loadColors(GtkWidget *widget) {
	if((g_list_length(colors)-1) < iSHASH("colorsize")) {
		g_warning("Unknown colorsize: %d\n", iSHASH("colorsize"));
		setBHASH("colorsize", 0);
	}

	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), iSHASH("colorsize"));
}

void loadSoundOptions(GtkWidget *widget) {
	if((g_list_length(sound_options)-1) < iSHASH("sound")) {
		g_warning("Unknown Sound-Flag: %d\n", iSHASH("sound"));
		setBHASH("sound", 0);
	}

	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), iSHASH("sound"));
}

void loadKeymap(GtkListStore *model) {
	DIR *kbdir = NULL;
	struct dirent *entry;
	GtkTreeIter iter;
	gint count = 0;
	GtkTreePath *path;
	gint ipath = 0, ipath_en = 1;

	/* fill the list */
	kbdir = opendir(KBPATH);
	if(kbdir == NULL) {
	        GtkWidget *dlg;

	        dlg = gtk_message_dialog_new(GTK_WINDOW(window_main),
	                                     GTK_DIALOG_DESTROY_WITH_PARENT,
	                                     GTK_MESSAGE_WARNING,
	                                     GTK_BUTTONS_CLOSE,
	                                     _("Unable to find keyboad "
	                                       "definitions: %s"), KBPATH);
	        gtk_dialog_run(GTK_DIALOG(dlg));
	        gtk_widget_destroy(dlg);
	}

	if(kbdir != NULL) {
		while((entry = readdir(kbdir)) != NULL) {
			if(entry->d_name[0] == '.')
				continue;
			if(!g_strcasecmp(entry->d_name, "common"))
				continue;
			if(!g_strcasecmp(entry->d_name, "modifiers"))
				continue;

			gtk_list_store_append(GTK_LIST_STORE(model), &iter);
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0,
				g_strdup(g_ascii_strup(entry->d_name, -1)), -1);

			count++;

			if(SHASH("keymap") != NULL) {
				if(!g_strcasecmp(SHASH("keymap"), entry->d_name))
					ipath = count;
			}
			if(!g_strcasecmp("EN", entry->d_name))
				ipath_en = count;
		}
	}

	if(kbdir != NULL)
		closedir(kbdir);

	if(count <= 0) {
		g_warning(_("Unable to find keyboad definitions: %s"),
			KBPATH);
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0,
			g_strdup("EN"), -1);
	}

	if(SHASH("keymap") != NULL) {
		path = gtk_tree_path_new_from_string(
			g_strdup_printf("%d", (ipath > 0) ? ipath-1 : ipath_en-1));
		if(path != NULL) {
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree_keymap),
				path, NULL, FALSE);
		}
	} else {
		path = gtk_tree_path_new_from_string(
			g_strdup_printf("%d", ipath_en-1));
		if(path != NULL) {
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree_keymap),
				path, NULL, FALSE);
		}
	}

	return;
}

void fillSoundOptions() {
	sound_options = g_list_append(sound_options, g_strdup(_("No sound output")));
	sound_options = g_list_append(sound_options, g_strdup(_("Play on this host")));
}

gint loadServers() {
	gchar **names;
	gint count, i;

	if(g_list_length(hostnames) != 0) {	/* empty the glist */
		while((count = g_list_length(hostnames) != 0)) {
			gchar *item;
			item = g_list_nth_data(hostnames, 0);
			hostnames = g_list_remove(hostnames, item);
		}
	}

	names = g_settings_get_strv(gcfg, "hostnames");
	count = g_strv_length(names);
	for(i=0; i<count; i++) {
		gchar *name;
		name = names[i];
		hostnames = g_list_append(hostnames, g_strdup(name));
	}
	g_strfreev(names);

	if(g_list_length(hostnames) == 0)
		return(1);
	return(0);
}

gint saveServers(gchar *newitem) {
	GList *walk, *tmplist = NULL;
	gchar **names;
	gint i, max=0;
	guint count;

	/* don't save empty lists */
	count = g_list_length(hostnames);
	if((count == 0) && (newitem == NULL))
		return(0);

	if(strlen(newitem) > 0) {	/* make sure the current hostname is first */
		max++;
		tmplist = g_list_append(tmplist, newitem);
	}

	for(i=0; i<count; i++) {
		gchar *item;
		item = g_list_nth_data(hostnames, i);

		if(!g_strncasecmp(item, newitem, MAXHOSTNAMELEN))
			continue;

		if(max >= MAXHOSTS)	/* store only max hosts */
			break;

		max++;
		tmplist = g_list_append(tmplist, item);
	}

	names = g_new0(gchar *, g_list_length(tmplist) + 1);
	for (i = 0, walk = tmplist; walk; walk = walk->next)
	        names[i++] = g_strdup(walk->data);

	names[i] = NULL;

	g_settings_set_strv(gcfg, "hostnames", (const gchar**)names);
	g_strfreev(names);

	return(0);
}

static void migrate_gconf_settings(const gchar *name) {
	gboolean needed = TRUE;
	GError *error = NULL;
	GKeyFile *kf;
	GtkWidget *dlg;
	GtkMessageType type;
	gchar **list;
	gchar *msg;
	gsize i, n;

	kf = g_key_file_new();

	g_key_file_load_from_data_dirs(kf, "gsettings-data-convert",
                                       NULL, G_KEY_FILE_NONE, NULL);
	list = g_key_file_get_string_list(kf, "State", "converted", &n, NULL);

	if (list) {
	        for (i = 0; i < n; i++)
	                if (!g_strcmp0(list[i], name)) {
	                        needed = FALSE;
	                        break;
	                }
	        g_strfreev(list);
	}
	g_key_file_free(kf);

	if (needed) {
	        g_spawn_command_line_sync("gsettings-data-convert",
	                                  NULL, NULL, NULL, &error);
	        if (error) {
	                msg = g_strdup_printf(
	                        _("Could not migrate old GConf settings: %s\n"
	                          "Please make sure that GConf is installed "
	                          "and the gsettings-data-convert tool is in "
	                          "your PATH.\nAlternatively, ignore this "
	                          "message and convert your old settings "
	                          "manually."), error->message);
                        type = GTK_MESSAGE_WARNING;
	                g_error_free(error);
	        } else {
	                msg = g_strdup(_("Old GConf settings were either "
	                                 "missing or migrated successfully."));
	                type = GTK_MESSAGE_INFO;

	                /* Allow some time for the GSettings backend
	                   to record the changes, otherwise the
	                   default values from the new schema might be
	                   set.  */
	                sleep(1);
	        }

	        dlg = gtk_message_dialog_new(GTK_WINDOW(window_main),
	                                     GTK_DIALOG_DESTROY_WITH_PARENT,
	                                     type, GTK_BUTTONS_CLOSE,
	                                     "%s", msg);
	        g_free(msg);
	        gconf_migration = dlg;
	}
}


gint loadOptions() {
	gchar *value;
	gint i;

	migrate_gconf_settings("grdesktop.convert");

	i = g_settings_get_boolean(gcfg, "bitmapupd");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("bitmapupd"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "clientname");
	g_hash_table_insert(config, g_strdup("clientname"), g_strdup(value));
	g_free(value);

	i = g_settings_get_int(gcfg, "colorsize");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("colorsize"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "username");
	g_hash_table_insert(config, g_strdup("username"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "password");
	g_hash_table_insert(config, g_strdup("password"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "savepw");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("savepw"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "domain");
	g_hash_table_insert(config, g_strdup("domain"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "geometry");
	g_hash_table_insert(config, g_strdup("geometry"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "hidewmdecoration");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("hidewmdecoration"),
	                    g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "showopts");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("showopts"), g_strdup(value));
	g_free(value);

	i = g_settings_get_int(gcfg, "rdp-protocol");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("rdp-protocol"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "keymap");
	g_hash_table_insert(config, g_strdup("keymap"), g_strdup(value));
	g_free(value);

	i = g_settings_get_int(gcfg, "sound");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("sound"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "program");
	g_hash_table_insert(config, g_strdup("program"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "ppath");
	g_hash_table_insert(config, g_strdup("ppath"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "runprog");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("runprog"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "motionevt");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("motionevt"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "noencryption");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("noencryption"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "wmkeybindings");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("wmkeybindings"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "attconsole");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("attconsole"), g_strdup(value));
	g_free(value);

	i = g_settings_get_boolean(gcfg, "usessh");
	value = g_strdup_printf("%d", i);
	g_hash_table_insert(config, g_strdup("usessh"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "sshuser");
	g_hash_table_insert(config, g_strdup("sshuser"), g_strdup(value));
	g_free(value);

	value = g_settings_get_string(gcfg, "sshhost");
	g_hash_table_insert(config, g_strdup("sshhost"), g_strdup(value));
	g_free(value);

	/* get the current username */
	if(!strlen(SHASH("username"))) {
		g_hash_table_insert(config, "username",
			g_strup((gchar*)g_get_user_name()));
	}

	return(0);
}

gint saveOptions() {
	/* Apply all changes at once.  */
	g_settings_delay(gcfg);

	if(SHASH("username") != NULL)
		g_settings_set_string(gcfg, "username", SHASH("username"));
	if((SHASH("password") != NULL) && (iSHASH("savepw") == TRUE))
		g_settings_set_string(gcfg, "password", SHASH("password"));
	if(SHASH("domain") != NULL)
		g_settings_set_string(gcfg, "domain", SHASH("domain"));
	g_settings_set_boolean(gcfg, "savepw", iSHASH("savepw"));
	g_settings_set_int(gcfg, "rdp-protocol", iSHASH("rdp-protocol"));
	if(SHASH("geometry") != NULL)
		g_settings_set_string(gcfg, "geometry", SHASH("geometry"));
	if(SHASH("keymap") != NULL)
		g_settings_set_string(gcfg, "keymap", SHASH("keymap"));
	g_settings_set_int(gcfg, "colorsize", iSHASH("colorsize"));
	g_settings_set_int(gcfg, "sound", iSHASH("sound"));
	if(SHASH("program") != NULL)
		g_settings_set_string(gcfg, "program", SHASH("program"));
	if(SHASH("ppath") != NULL)
		g_settings_set_string(gcfg, "ppath", SHASH("ppath"));
	g_settings_set_boolean(gcfg, "runprog", iSHASH("runprog"));
	g_settings_set_boolean(gcfg, "bitmapupd", iSHASH("bitmapupd"));
	g_settings_set_boolean(gcfg, "motionevt", iSHASH("motionevt"));
	g_settings_set_boolean(gcfg, "hidewmdecoration",
		iSHASH("hidewmdecoration"));
	g_settings_set_boolean(gcfg, "noencryption", iSHASH("noencryption"));
	g_settings_set_boolean(gcfg, "wmkeybindings", iSHASH("wmkeybindings"));
	g_settings_set_boolean(gcfg, "attconsole", iSHASH("attconsole"));
	if(SHASH("clientname") != NULL)
		g_settings_set_string(gcfg, "clientname", SHASH("clientname"));
	g_settings_set_boolean(gcfg, "usessh", iSHASH("usessh"));
	if(SHASH("sshuser") != NULL)
		g_settings_set_string(gcfg, "sshuser", SHASH("sshuser"));
	if(SHASH("sshhost") != NULL)
		g_settings_set_string(gcfg, "sshhost", SHASH("sshhost"));
	g_settings_set_boolean(gcfg, "showopts", iSHASH("showopts"));

	g_settings_apply(gcfg);

	return(0);
}

gchar *screenCommand() {
	if(!g_strcasecmp(SHASH("geometry"), _("Fullscreen"))) {
		return("-f ");
	}

	return(g_strdup_printf("-g %s ", SHASH("geometry")));
}

gchar *colorCommand() {
	int item;

	switch(iSHASH("colorsize")) {
	case 0:
		item = 8;
		break;
	case 1:
		item = 15;
		break;
	case 2:
		item = 16;
		break;
	case 3:
		item = 24;
		break;
	default:
		item = 8;
	}

	return(g_strdup_printf("-a %d ", item));
}

gchar *keymapCommand() {
	return(g_strdup_printf("-k %s ", SHASH("keymap")));
}

void fill_dialog() {
	gchar *licence_path = NULL;
	DIR *licence_dir = NULL;

	if(SHASH("hostname") != NULL) {
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo_host)->entry),
			SHASH("hostname"));
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo_host2)->entry),
			SHASH("hostname"));
	}

	if(SHASH("username") != NULL)
		gtk_entry_set_text(GTK_ENTRY(input_username), SHASH("username"));
	if(SHASH("password") != NULL)
		gtk_entry_set_text(GTK_ENTRY(input_password), SHASH("password"));
	if(SHASH("domain") != NULL)
		gtk_entry_set_text(GTK_ENTRY(input_domain), SHASH("domain"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_savepw),
		iSHASH("savepw"));

	loadRdpProtocols(menu_rdp_proto);

	loadScreens(GTK_ADJUSTMENT(adj_screensize));
	sig_scroll(GTK_WIDGET(scroll_screensize), NULL);

	loadColors(menu_colorsize);
	sig_colchange(NULL, NULL);

	loadSoundOptions(menu_sound);
	sig_sound(NULL, NULL);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_prog),
		iSHASH("runprog"));
	if(SHASH("program") != NULL)
		gtk_entry_set_text(GTK_ENTRY(input_program), SHASH("program"));
	if(SHASH("ppath") != NULL)
		gtk_entry_set_text(GTK_ENTRY(input_ppath), SHASH("ppath"));
	gtk_widget_set_sensitive(input_program, iSHASH("runprog"));
	gtk_widget_set_sensitive(input_ppath, iSHASH("runprog"));

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_bitmapupd),
		iSHASH("bitmapupd"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_motionevt),
		iSHASH("motionevt"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_hidewmdecoration),
		iSHASH("hidewmdecoration"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_noencr),
		iSHASH("noencryption"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_wmkeybindings),
		iSHASH("wmkeybindings"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_attconsole),
		iSHASH("attconsole"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_sshopts),
		iSHASH("usessh"));
	gtk_widget_set_sensitive(btn_sshopts, iSHASH("usessh"));

	licence_path = g_build_filename(g_get_home_dir(), ".rdesktop", NULL);
	licence_dir = opendir(licence_path);
	if(licence_dir != NULL) {
		struct dirent *entry = NULL;
		const gchar *prefix = "licence.";
		GList *licence_list = NULL;

		while((entry = readdir(licence_dir)) != NULL) {
			gchar *fullname = NULL;
			gchar *hostname = &entry->d_name[strlen(prefix)];
			struct stat sts;

			fullname = g_strdup_printf("%s/%s", licence_path,
				entry->d_name);

			if(stat(fullname, &sts) == -1)
				continue;
			if(!S_ISREG(sts.st_mode))
				continue;
	
			if(strlen(entry->d_name) <= strlen(prefix))
				continue;

			licence_list = g_list_append(licence_list, g_strdup(hostname));
		}
		closedir(licence_dir);
		g_free(licence_path);
		if(licence_list != NULL) {
			gtk_combo_set_popdown_strings(GTK_COMBO(combo_clientname),
				licence_list);
		}
	}

	if(SHASH("clientname") != NULL) {
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo_clientname)->entry),
			SHASH("clientname"));
	}

	/* finally, we deactivate all rdp unsupported widgets */
	sig_rdp_protocol(NULL, NULL);
}

